using System;
using System.Collections.Generic;
using System.Text;
using System.Web.UI.HtmlControls;
using System.Web.UI.WebControls;
using System.Web.UI;
using System.Web.UI.Design;
using System.Collections.Specialized;
using System.Globalization;
using System.Web;
using AjaxControlToolkit;

[assembly: WebResource("AjaxSet.TextEditList.TextEditList.js", "text/javascript")]

namespace AjaxSet
{
	public class TextEditList : ListControl, INamingContainer, IPostBackDataHandler, IScriptControl //IRepeatInfoUser, 
	{
		private bool _cachedIsEnabled;
		private TextBox _controlToRepeat;
		private ScriptManager sm;
		private ListItemCollection items;
		private bool _itemsProcessedAfterSubmit = false;

		public TextEditList()
		{
		}
		protected override Style CreateControlStyle()
		{
			return new TableStyle(this.ViewState);
		}
		protected override Control FindControl(string id, int pathOffset)
		{
			return this;
		}
		protected virtual Style GetItemStyle(ListItemType itemType, int repeatIndex)
		{
			return null;
		}
		private bool _hasNotifiedOfChange;
		protected virtual bool LoadPostData(string postDataKey, NameValueCollection postCollection)
		{
			string name = postDataKey.Substring(this.UniqueID.Length + 1);
			if (name == "addField")
				return false;
			int index = int.Parse(name, CultureInfo.InvariantCulture);
			this.EnsureDataBound();
			if ((index >= 0) && (index < this.Items.Count))
			{
				ListItem item = this.Items[index];
				if (!item.Enabled)
					return false;
				string presentValue = item.Text;
				string postedValue = postCollection[postDataKey];
				if (presentValue == null || !presentValue.Equals(postedValue))
				{
					item.Text = postedValue;
					if (!this._hasNotifiedOfChange)
					{
						this._hasNotifiedOfChange = true;
						return true;
					}
				}
			}
			else
			{
				Items.Add(new ListItem(postCollection[postDataKey], string.Empty));
				return true;
			}
			return false;
		}
		protected virtual void RaisePostDataChangedEvent()
		{
			this.OnSelectedIndexChanged(EventArgs.Empty);
		}
		protected override void AddAttributesToRender(HtmlTextWriter writer)
		{
			base.AddAttributesToRender(writer);
		}
		protected override void OnPreRender(EventArgs e)
		{
			if (!this.DesignMode)
			{
				// Test for ScriptManager and register if it exists
				sm = ScriptManager.GetCurrent(Page);
				if (sm == null)
					throw new HttpException("A ScriptManager control must exist on the current page.");
				sm.RegisterScriptControl(this);
			}
			base.OnPreRender(e);
			this.ControlToRepeat.AutoPostBack = this.AutoPostBack;
			this.ControlToRepeat.CausesValidation = this.CausesValidation;
			this.ControlToRepeat.ValidationGroup = this.ValidationGroup;
			if (this.Page != null)
			{
				for (int i = 0; i < this.Items.Count; i++)
				{
					this._controlToRepeat.ID = i.ToString(NumberFormatInfo.InvariantInfo);
					this.Page.RegisterRequiresPostBack(this._controlToRepeat);
				}
			}
		}
		public override void RenderBeginTag(HtmlTextWriter writer)
		{
			AddAttributesToRender(writer);
			writer.RenderBeginTag(HtmlTextWriterTag.Table);
		}
		public override void RenderEndTag(HtmlTextWriter writer)
		{
			writer.RenderEndTag();
		}
		protected override void RenderContents(HtmlTextWriter writer)
		{
			if (!this.DesignMode)
				sm.RegisterScriptDescriptors(this);
			if ((this.Items.Count != 0))
			{
				Style style = base.ControlStyleCreated ? base.ControlStyle : null;
				short num = this.TabIndex;
				bool flag = false;
				this.ControlToRepeat.TabIndex = num;
				if (num != 0)
				{
					if (!this.ViewState.IsItemDirty("TabIndex"))
						flag = true;
					this.TabIndex = 0;
				}
				RenderTableLayout(writer);
				if (this.Page != null)
					this.Page.ClientScript.RegisterForEventValidation(this.UniqueID);
				if (num != 0)
					this.TabIndex = num;
				if (flag)
					this.ViewState.SetItemDirty("TabIndex", false);
			}
		}
		protected void RenderTableLayout(HtmlTextWriter writer)
		{
			writer.RenderBeginTag(HtmlTextWriterTag.Tbody);
			for (int i = 0; i < this.Items.Count; i++)
				RenderTableItem(Items[i], i, writer);
			writer.RenderEndTag();
		}
		protected virtual void RenderTableItem(ListItem item, int itemIndex, HtmlTextWriter writer)
		{
			if (itemIndex == 0)
				this._cachedIsEnabled = base.IsEnabled;

			if (item.Value == "NewItem")
			{
				if (!string.IsNullOrEmpty(NewItemRowCssClass))
					writer.AddAttribute(HtmlTextWriterAttribute.Class, this.NewItemRowCssClass);
			}
			else
			{
				if (!string.IsNullOrEmpty(RowCssClass))
					writer.AddAttribute(HtmlTextWriterAttribute.Class, this.RowCssClass);
			}
			writer.RenderBeginTag(HtmlTextWriterTag.Tr);

			if (!string.IsNullOrEmpty(FieldCssClass))
				writer.AddAttribute(HtmlTextWriterAttribute.Class, this.FieldCssClass);
			writer.RenderBeginTag(HtmlTextWriterTag.Td);

			TextBox textBox = this.ControlToRepeat;
			textBox.Attributes.Clear();
			if (item.Value == "NewItem")
				textBox.ID = "addField";
			else
				textBox.ID = itemIndex.ToString(NumberFormatInfo.InvariantInfo);
			textBox.Text = item.Text;
			textBox.Enabled = this._cachedIsEnabled && item.Enabled;
			if (MaxLength > 0)
				textBox.MaxLength = MaxLength;
			textBox.RenderControl(writer); //field

			writer.RenderEndTag();   //end op td

			if (!string.IsNullOrEmpty(ButtonCssClass))
				writer.AddAttribute(HtmlTextWriterAttribute.Class, this.ButtonCssClass);
			writer.RenderBeginTag(HtmlTextWriterTag.Td);

			if (LinkButton)
			{
				HtmlAnchor button = new HtmlAnchor();
				if (item.Value == "NewItem")
				{
					button.InnerText = AddButtonText;
					button.HRef = "#";
					button.ID = this.UniqueID + "_addbtn";
				}
				else
				{
					button.InnerText = DeleteButtonText;
					button.HRef = "#";
					button.ID = this.UniqueID + "_delbtn_" + itemIndex.ToString(NumberFormatInfo.InvariantInfo);
				}
				button.Disabled = !(this._cachedIsEnabled && item.Enabled);
				button.RenderControl(writer);
			}
			if (!LinkButton)
			{
				Button button = new Button();
				if (item.Value == "NewItem")
				{
					button.Text = AddButtonText;
					button.ID = this.UniqueID + "_addbtn";
				}
				else
				{
					button.Text = DeleteButtonText;
					button.ID = this.UniqueID + "_delbtn_" + itemIndex.ToString(NumberFormatInfo.InvariantInfo);
				}
				button.Enabled = this._cachedIsEnabled && item.Enabled;
				button.RenderControl(writer);
			}

			writer.RenderEndTag(); //end of td
			writer.RenderEndTag();
		}
		protected virtual bool HasFooter
		{
			get { return false; }
		}

		protected virtual bool HasHeader
		{
			get { return false; }
		}

		protected virtual bool HasSeparators
		{
			get { return false; }
		}
		protected virtual int RepeatedItemCount
		{
			get
			{
				if (this.Items == null)
					return 0;
				return this.Items.Count;
			}
		}
		public virtual RepeatLayout RepeatLayout
		{
			get
			{
				object currentValue = this.ViewState["RepeatLayout"];
				if (currentValue != null)
				{
					return (RepeatLayout)currentValue;
				}
				return RepeatLayout.Table;
			}
			set
			{
				if ((value < RepeatLayout.Table) || (value > RepeatLayout.Flow))
				{
					throw new ArgumentOutOfRangeException("value");
				}
				this.ViewState["RepeatLayout"] = value;
			}
		}
		public string DeleteConfirmMessage
		{
			get
			{
				if (this.ViewState["DeleteConfirmMessage"] == null)
					return "Delete?";
				return (string)this.ViewState["DeleteConfirmMessage"];
			}
			set
			{
				this.ViewState["DeleteConfirmMessage"] = value;
			}
		}
		public bool LinkButton
		{
			get
			{
				if (this.ViewState["LinkButton"] == null)
					return false;
				return (bool)this.ViewState["LinkButton"];
			}
			set
			{
				this.ViewState["LinkButton"] = value;
			}
		}
		public string DeleteButtonText
		{
			get
			{
				if (this.ViewState["DeleteButtonText"] == null)
					return "Delete";
				return (string)this.ViewState["DeleteButtonText"];
			}
			set
			{
				this.ViewState["DeleteButtonText"] = value;
			}
		}

		public string AddButtonText
		{
			get
			{
				if (this.ViewState["AddButtonText"] == null)
					return "Add";
				return (string)this.ViewState["AddButtonText"];
			}
			set
			{
				this.ViewState["AddButtonText"] = value;
			}
		}
		public string FieldCssClass
		{
			get
			{
				if (this.ViewState["FieldCssClass"] == null)
					return string.Empty;
				return (string)this.ViewState["FieldCssClass"];
			}
			set
			{
				this.ViewState["FieldCssClass"] = value;
			}
		}
		public string ButtonCssClass
		{
			get
			{
				if (this.ViewState["ButtonCssClass"] == null)
					return string.Empty;
				return (string)this.ViewState["ButtonCssClass"];
			}
			set
			{
				this.ViewState["ButtonCssClass"] = value;
			}
		}
		public string RowCssClass
		{
			get
			{
				if (this.ViewState["RowCssClass"] == null)
					return string.Empty;
				return (string)this.ViewState["RowCssClass"];
			}
			set
			{
				this.ViewState["RowCssClass"] = value;
			}
		}
		public string NewItemRowCssClass
		{
			get
			{
				if (this.ViewState["NewItemRowCssClass"] == null)
					return string.Empty;
				return (string)this.ViewState["NewItemRowCssClass"];
			}
			set
			{
				this.ViewState["NewItemRowCssClass"] = value;
			}
		}
		public int MaxLength
		{
			get
			{
				if (this.ViewState["MaxLength"] == null)
					return -1;
				return (int)this.ViewState["MaxLength"];
			}
			set
			{
				this.ViewState["MaxLength"] = value;
			}
		}

		public override ListItemCollection Items
		{
			get
			{
				this.items = base.Items;
				if (this.Initialized)
				{
					if (!_itemsProcessedAfterSubmit && Page != null && Page.IsPostBack)
					{
						for (int i = this.items.Count - 1; i >= 0; i--)
						{
							if (this.items[i].Value == "NewItem")
								continue;
							string formFieldName = this.UniqueID + "$" + i.ToString();
							if (Page.Request != null)
								if (Page.Request.Form[formFieldName] == null)
									this.items.RemoveAt(i);
						}
						_itemsProcessedAfterSubmit = true;
					}
				}
				else
				{
					if (this.items.FindByValue("NewItem") != null)
						this.items.Remove(this.items.FindByValue("NewItem"));
				}
				return this.items;
			}
		}
		protected override void CreateChildControls()
		{
			base.CreateChildControls();
			if (this.items.FindByValue("NewItem") == null)
				this.items.Add(new ListItem(string.Empty, "NewItem"));
		}
		public override void DataBind()
		{
			base.DataBind();
		}

		protected virtual IEnumerable<ScriptDescriptor> GetScriptDescriptors()
		{
			ScriptControlDescriptor descriptor = new ScriptControlDescriptor("AjaxSet.TextEditList", this.ClientID);
			string[] itemsArray = new string[Items.Count];
			for (int i = 0; i < Items.Count; i++)
				itemsArray[i] = Items[i].Text;
			descriptor.AddProperty("items", itemsArray);
			descriptor.AddProperty("layout", this.RepeatLayout.ToString());
			descriptor.AddProperty("deleteConfirm", this.DeleteConfirmMessage);
			descriptor.AddProperty("isLinkButton", this.LinkButton);
			descriptor.AddProperty("deleteButtonText", this.DeleteButtonText);
			descriptor.AddProperty("addButtonText", this.AddButtonText);
			descriptor.AddProperty("fieldCssClass", this.FieldCssClass);
			descriptor.AddProperty("buttonCssClass", this.ButtonCssClass);
			descriptor.AddProperty("rowCssClass", this.RowCssClass);
			descriptor.AddProperty("newItemRowCssClass", this.NewItemRowCssClass);
			descriptor.AddProperty("maxLength", this.MaxLength);
			return new ScriptDescriptor[] { descriptor };
		}

		protected virtual IEnumerable<ScriptReference> GetScriptReferences()
		{
			ScriptReference reference = new ScriptReference("AjaxSet.TextEditList.TextEditList.js", "AjaxSet");
			return new ScriptReference[] { reference };
		}
		private TextBox ControlToRepeat
		{
			get
			{
				if (this._controlToRepeat == null)
				{
					this._controlToRepeat = new TextBox();
					this._controlToRepeat.EnableViewState = true;
					this.Controls.Add(this._controlToRepeat);
					//					this._controlToRepeat.AutoPostBack = this.AutoPostBack;
					this._controlToRepeat.CausesValidation = this.CausesValidation;
					this._controlToRepeat.ValidationGroup = this.ValidationGroup;
				}
				return this._controlToRepeat;
			}
		}

		#region IPostBackDataHandler Members

		bool IPostBackDataHandler.LoadPostData(string postDataKey, System.Collections.Specialized.NameValueCollection postCollection)
		{
			return this.LoadPostData(postDataKey, postCollection);
		}

		void IPostBackDataHandler.RaisePostDataChangedEvent()
		{
			this.RaisePostDataChangedEvent();
		}

		#endregion

		#region IScriptControl Members

		IEnumerable<ScriptDescriptor> IScriptControl.GetScriptDescriptors()
		{
			return GetScriptDescriptors();
		}

		IEnumerable<ScriptReference> IScriptControl.GetScriptReferences()
		{
			return GetScriptReferences();
		}
		#endregion
	}
}
