using System;
using System.Web.UI.WebControls;
using System.Web.UI;
using System.ComponentModel;
using System.ComponentModel.Design;
using AjaxControlToolkit;
using System.Diagnostics.CodeAnalysis;

[assembly: WebResource("AjaxSet.Validator.ValidatorBehavior.js", "text/javascript")]

namespace AjaxSet
{
	[Designer(typeof(ValidatorDesigner))]
	[ClientScriptResource("AjaxSet.ValidatorBehavior", "AjaxSet.Validator.ValidatorBehavior.js")]
	[TargetControlType(typeof(Control))]
	public class ValidatorExtender : ExtenderControlBase
	{
		/// <summary>
		/// CSS class used to indicate that control is invalid
		/// </summary>
		[DefaultValue("")]
		[ExtenderControlProperty()]
		public string ErrorCssClass
		{
			get
			{
				return GetPropertyValue<string>("ErrorCssClass", "");
			}
			set
			{
				SetPropertyValue<string>("ErrorCssClass", value);
			}
		}

		/// <summary>
		/// The validation method name to call
		/// </summary>
		[DefaultValue("")]
		[SuppressMessage("Microsoft.Globalization", "CA1303:DoNotPassLiteralsAsLocalizedParameters", Justification = "Assembly is not localized")]
		[ExtenderControlProperty()]
		public string ServiceMethod
		{
			get
			{
				return GetPropertyValue<string>("ServiceMethod", "");
			}
			set
			{
				SetPropertyValue<string>("ServiceMethod", value);
			}
		}
		/// <summary>
		/// The path of the web service to call, or if this is blank, a page method will be
		/// called instead.
		/// </summary>        
		[ExtenderControlProperty()]
		[UrlProperty()]
		[TypeConverter(typeof(ServicePathConverter))]
		public string ServicePath
		{
			get
			{

				return GetPropertyValue<string>("ServicePath", "");
			}
			set
			{
				SetPropertyValue<string>("ServicePath", value);
			}
		}

		private bool ShouldSerializeServicePath()
		{
			return !String.IsNullOrEmpty(ServiceMethod);
		}
		/// <summary>
		/// A CSS style to apply while the validation is in progress
		/// </summary>
		[DefaultValue("")]
		[ExtenderControlProperty()]
		public string UpdatingCssClass
		{
			get
			{
				return GetPropertyValue<string>("UpdatingCssClass", "");
			}
			set
			{
				SetPropertyValue<string>("UpdatingCssClass", value);
			}
		}

		protected override void OnPreRender(EventArgs e)
		{
			base.OnPreRender(e);
			if (!Page.ClientScript.IsStartupScriptRegistered("ValidatorExtender"))
			{
				string script = "var __AtlasValidatorExtendersInProgress = 0;\n" +
												"var __AtlasValidatorExtendersInvalid = 0;\n" +
												"var __Submiting = false;\n" +
												"\n" +
												"formSubmitMockup = function() {\n" +
												"  var originalSubmit = WebForm_OnSubmit();\n" +
												"  if (__AtlasValidatorExtendersInProgress <= 0) {\n" +
												"    if (__AtlasValidatorExtendersInvalid > 0 || !originalSubmit) {\n" +
												"      __AtlasValidatorExtendersInProgress = 0;\n" +
												"      __AtlasValidatorExtendersInvalid = 0;\n" +
												"      return false;\n" +
												"    } else {\n" +
												"      theForm.onsubmit = null;\n" +
												"      theForm.submit();\n" +
												"    }\n" +
												"  } else {\n" +
												"    return false;\n" +
												"  }\n" +
												"}\n" +
												"theForm.onsubmit = formSubmitMockup;\n";
				Page.ClientScript.RegisterStartupScript(this.GetType(), "ValidatorExtender", script, true);
			}
		}
	}
}
