Type.registerNamespace('AjaxSet');

Validator.ValidatorBehavior = function(element) {
  Validator.ValidatorBehavior.initializeBase(this, [element]);
  this._errorCssClass;  //error css class for fields
  this._updatingCssClass;  //updating css class for fields
  this._serviceMethod = null;  //web method with validation
  this._servicePath = null;
  this._callID = 0;
  this._currentCallID = -1;
  this._controlValue;
}

Validator.ValidatorBehavior.prototype = {
  initialize : function() {
    Validator.ValidatorBehavior.callBaseMethod(this, 'initialize');
    //store original validation function and replace it by extender mockup
    this.get_element().defaultEvaluationfunction = this.get_element().evaluationfunction;
    this.get_element().evaluationfunction = Function.createDelegate(this, this.evaluationfunctionMockup);
    //store original error message from the validator and clear the validator content
    this.get_element().defaultErrorMessage = this.get_element().innerText;
    this.get_element().style.display = "none";
    //store original css class from the field assigned to validator
    var controltovalidate = $get(this.get_element().controltovalidate);
    controltovalidate.defultClassName = controltovalidate.className;    
    controltovalidate.onchange = null;
  },

  dispose : function() {
    Validator.ValidatorBehavior.callBaseMethod(this, 'dispose');
  },

  // property accessors here
  get_ErrorCssClass : function() {
    return this._errorCssClass;
  },

  set_ErrorCssClass : function(value) {
    this._errorCssClass = value;
  },

  get_ServicePath : function() {
    return this._servicePath;
  },
  
  set_ServicePath : function(value) {
    if (this._servicePath != value) {
        this._servicePath = value;
        this.raisePropertyChanged('ServicePath');
    }
  },  
  
  get_ServiceMethod : function() {
    return this._serviceMethod;
  },
  set_ServiceMethod : function(value) {
    if (this._serviceMethod != value) {
      this._serviceMethod = value;
      this.raisePropertyChanged('ServiceMethod');
    }
  },

  get_UpdatingCssClass : function() {
    return this._updatingCssClass;
  },

  set_UpdatingCssClass : function(value) {
    this._updatingCssClass = value;
  },
  
  //other methods
  validate : function(sender, args) {
    this._currentCallID = ++this._callID;
    if (this._servicePath && this._serviceMethod) {
      this._setValidating();
      //increase counter
      __AtlasValidatorExtendersInvalid ++;
      __AtlasValidatorExtendersInProgress ++;
      __Submiting = true;
      // Call the helper web service
      var svr = new Sys.Net.ServiceMethodRequest();
      svr.set_url(this._servicePath);
      svr.set_methodName(this._serviceMethod);
//      svr.get_parameters()["contextKey"] = "sd";
      svr.get_parameters()["value"] = args.Value;
      svr.get_parameters()["isValid"] = args.IsValid;
      svr.add_completed(Function.createDelegate(this, this._onMethodComplete));
      svr.add_error(Function.createDelegate(this, this._onMethodError));
      svr.add_timeout(Function.createDelegate(this, this._onMethodTimeout));
      svr.invoke(this._currentCallID);      
    }        
  },

  _onMethodComplete : function (sender, eventArgs) {
    if (sender.get_userContext() != this._currentCallID) return;
    // Time has passed; make sure the element is still accessible
    var e = this.get_element();
    if (e) {
      __AtlasValidatorExtendersInProgress--;
      __Submiting = (__AtlasValidatorExtendersInProgress > 0);
      if (sender.get_result() == "") 
      {
        //decrease counter if success
        __AtlasValidatorExtendersInvalid--;
        this._setNormal();
      }
      else 
      {
        this._setError(sender.get_result());
      }
      formSubmitMockup();
    }
  },

  _onMethodError : function(sender, eventArgs) {
    // ignore if it's not the current call.
    if (sender.get_userContext() != this._currentCallID) return;
    var e = this.get_element();
    var result = sender.get_result();
    if(result.get_httpStatusCode) {        
      this._setError("WebService call failed: " + (result.get_httpStatusCode()));    
    } else {
      this._setError(result.get_message());
    }
  },

  _onMethodTimeout : function(request, userContext) {
    // ignore if it's not the current call.
    //
    if (sender.get_userContext() != this._currentCallID) return;
    this._setError("Webservice call timed out.");
  },   
  
  _setNormal : function() {
      var controltovalidate = $get(this.get_element().controltovalidate);
      controltovalidate.disabled = false;
      controltovalidate.className = controltovalidate.defultClassName;
      controltovalidate.title = "";
      controltovalidate.value = this._controlValue;
      this.get_element().isvalid = true;
  },

  _setValidating : function() {
      var controltovalidate = $get(this.get_element().controltovalidate);
      controltovalidate.disabled = true;
      controltovalidate.className = this._updatingCssClass;
      this._controlValue = controltovalidate.value;
      controltovalidate.value = "";
      controltovalidate.title = "Validation...";
  },

  _setError : function(message) {
      var controltovalidate = $get(this.get_element().controltovalidate);
      controltovalidate.disabled = false;
      controltovalidate.className = this._errorCssClass;
      controltovalidate.title = message;
      controltovalidate.value = this._controlValue;
      this.get_element().isvalid = false;
  },
  
  evaluationfunctionMockup : function(val) {
    var value = "";
    var isValid = val.defaultEvaluationfunction(val); //call original validation function
    if (typeof(this.get_element().controltovalidate) == "string")
      value = ValidatorGetValue(this.get_element().controltovalidate);
    var controltovalidate = $get(this.get_element().controltovalidate);
    //set visual style for assigned field according to validation result
    if (!isValid) { 
      controltovalidate.className = this._errorCssClass;
      controltovalidate.title = this.get_element().defaultErrorMessage;
    } 
    else 
    {
      if (this._servicePath && this._serviceMethod) {
        var args = {Value:value, IsValid:isValid};
        this.validate(val, args);
        isValid = false;
      } 
      else 
      {     
        controltovalidate.className = controltovalidate.defultClassName;
        controltovalidate.title = "";
      }
    }
    return isValid;  //return validation result for further processing
  }  
}

Validator.ValidatorBehavior.registerClass('Validator.ValidatorBehavior', AjaxControlToolkit.BehaviorBase);
